package cc.smtweb.system.bpm.web.engine.model.listcard.abshandler;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwConsts;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.exception.SwException;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.dynPage.AbstractDynPageHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageListHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynRetBean;

/**
 * Created by Akmm at 2022-08-16 15:43
 */
public class ModelLoadHandler<T extends DefaultEntity> extends AbstractDynPageHandler {
    protected T bean;
    protected String cardDsName = ModelHelper.DATASET_NAME_CARD;

    //新增卡片操作，初始化定义的数据集
    public R add() {
        SwMap ret = new SwMap();
        bean = addBean(ret, cardDsName);
        afterAdd(ret);
        return R.success(ret);
    }

    //创建一个bean，并加到返回对象中
    protected <E extends DefaultEntity> E addBean(SwMap ret, String dsName) {
        //返回的数据，以dataset.name为key，查出的结果（bean或list）为value
        PageDataset cardDataset = findCardDataset(dsName);
        ModelTable table = cardDataset._getModelTable();
        if (table == null) throw new SwException("卡片主表未找到！");
        final EntityDao<E> dao = DbEngine.getInstance().findDao(table.getName());

        E entity = dao.createBean(us.getUserId());
        entity.setEntityId(DbEngine.getInstance().nextId());

        ret.put(dsName, DynRetBean.createBean(entity));
        return entity;
    }


    //新增初始化 - bean
    protected void afterAdd(SwMap ret) {
    }

    public R load() {
        SwMap ret = new SwMap();
        //过滤条件
        SwMap filter = params.readMap("filter");
        load(ret, filter);
        return R.success(ret);
    }

    //从数据库加载bean，并设到返回值
    protected void load(SwMap ret, SwMap filter) {
        PageDataset cardDataset = findCardDataset(cardDsName);
        bean = provider.loadBean(cardDataset, filter.readLong(SwConsts.NAME_ID));
        ret.put(cardDsName, DynRetBean.createBean(bean));
    }

    //给子类使用的工具方法：根据数据集配置，读取bean，并加到返回值
    protected <E extends DefaultEntity> E loadBean(SwMap ret, String dsName, SwMap filter) {
        PageDataset cardDataset = findCardDataset(dsName);
        E bean = provider.loadBean(cardDataset, filter);
        ret.put(dsName, DynRetBean.createBean(bean));
        return bean;
    }


    protected DynPageListHandler getListWorker(SwMap filter, PageDataset pageDataSet) {
        DynPageListHandler listHandler = new DynPageListHandler(form.getId(), filter, pageDataSet);
        listHandler.init(params, us);
        return listHandler;
    }

    /**
     * 列表数据
     *
     * @return
     */
    protected R list(String dsName) {
        //过滤条件
        SwMap filter = params.readMap("filter");
        //对应的数据集定义
        PageDataset pageDataSet = findListDataset(dsName);
        DynRetBean bean = DynRetBean.createList(getListWorker(filter, pageDataSet).buildListData());
        return R.success(bean);
    }

    protected R getTotal(String dsName) {
        //数据集
        PageDataset pageDataSet = findListDataset(dsName);
        //过滤条件
        SwMap filter = params.readMap("filter");

        return getListWorker(filter, pageDataSet).getTotal();
    }

    public R list() {
        return list(ModelHelper.DATASET_NAME_LIST);
    }

    /**
     * 计算分页数据
     *
     * @return
     */
    public R getTotal() {
        return getTotal(ModelHelper.DATASET_NAME_LIST);
    }
}
