package cc.smtweb.system.bpm.web.sys.user.role;

import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlan;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanCache;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanContent;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanItem;
import cc.smtweb.system.bpm.web.sys.user.user.User;
import cc.smtweb.system.bpm.web.sys.user.user.UserCache;
import cc.smtweb.system.bpm.web.sys.user.user.UserRoleCache;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @Author: tanghp
 * @Date: 2022-09-06 18:35
 * @Desc: 角色辅助类
 */
public final class RoleHelper {
    public static boolean isAdmin(long userId){
        if(userId ==1L) return true;
       User user = UserCache.getInstance().get(userId);
       if(user!=null){
           return "admin".equalsIgnoreCase(user.getNicky());
       }
       return false;
    }
    /**
     * 获取角色权限
     * @param userId 用户ID
     * @param menuPlanId 菜单方案ID
     * @return
     */
    public static List<RoleRight> getRoleRightList(long userId, long menuPlanId){
        return null;
    }
    /**
     * 获取用户的菜单权限
     * @param userId 用户ID
     * @param partyId 机构ID
     * @param incParent 是否包含父节点，（子节点配置了，把父节点也查出来）
     * @return
     */
    public static Set<Long> getRoleMenuIds(long userId,long partyId,boolean incParent){
        Set<Long> mpIds = getMenuPlanIds(userId,partyId);
        if(mpIds.size()==0){
            return  new HashSet<>();
        }
        return getRoleMenuIds(userId,partyId,mpIds.iterator().next(),incParent);
    }
    /**
     * 获取用户的菜单权限
     * @param userId 用户ID
     * @param partyId 机构ID
     * @param menuPlanId 菜单方案ID
     * @param incParent 是否包含父节点，（子节点配置了，把父节点也查出来）
     * @return
     */
    public static Set<Long>  getRoleMenuIds(long userId,long partyId, long menuPlanId,boolean incParent){
        Set<Long> menuIds = new HashSet<>();
        MenuPlan menuPlan = MenuPlanCache.getInstance().get(menuPlanId);
        if(menuPlan==null)return menuIds;
        Set<Long> roleIds = UserRoleCache.getInstance().getRoleIdByUP(userId,partyId);
        RoleCache roleCache = RoleCache.getInstance();
        roleIds.forEach((roleId)-> {
            Role role = roleCache.get(roleId);
            if(role==null || role.getSmpId()!=menuPlanId){
                return;
            }
            RoleRightContent roleRightContent = new RoleRightContent(role.getPrivilege());
            menuIds.addAll(roleRightContent.getMenuRightIds());
        });
        MenuPlanContent menuPlanContent = new MenuPlanContent(menuPlan.getContent());
        if(incParent){
            Set<Long> tempIds = new HashSet<>(menuIds);
            tempIds.forEach(id->{
                fillMenuId(id,menuPlanContent,menuIds);
            });
        }
        return menuIds;
    }
    private static void fillMenuId(Long id,MenuPlanContent menuPlanContent, Set<Long> menuIds){
        menuIds.add(id);
        MenuPlanItem parent = menuPlanContent.findParentById(id);
        if(parent!=null){
            if(!menuIds.contains(parent.getId())){
                fillMenuId(parent.getId(),menuPlanContent,menuIds);
            }
        }
    }
    /**
     * 获取用户的菜单方案
     * @param userId 用户ID
     * @param partyId 机构ID
     * @return 菜单方案ID
     */
    public static Set<Long> getMenuPlanIds(long userId,long partyId){
        Set<Long> mpList = new HashSet<>();
        Set<Long> roleIds = UserRoleCache.getInstance().getRoleIdByUP(userId,partyId);
        RoleCache roleCache = RoleCache.getInstance();
        roleIds.forEach((roleId)-> {
            Role role = roleCache.get(roleId);
            if(role!=null&&role.getSmpId()>0L){
                mpList.add(role.getSmpId());
            }
        });
        return mpList;
    }
    /**
     * 获取用户的菜单方案
     * @param userId 用户ID
     * @param partyId 机构ID
     * @return 菜单方案
     */
    public static Set<MenuPlan> getMenuPlans(long userId,long partyId){
        Set<MenuPlan> mpList = new HashSet<>();
        Set<Long> roleIds = UserRoleCache.getInstance().getRoleIdByUP(userId,partyId);
        roleIds.forEach((roleId)-> {
            Role role =  RoleCache.getInstance().get(roleId);
            if(role!=null&&role.getSmpId()>0L){
                MenuPlan mp = MenuPlanCache.getInstance().get(role.getSmpId());
                if(mp!=null){
                    mpList.add(mp);
                }
            }
        });
        return mpList;
    }
}
